#include "config.h"
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include <string.h>
#ifdef HAVE_IMLIB
#include <Imlib.h>
#endif
#ifdef HAVE_XM_XPMP_H
#include <Xm/XpmP.h>
#elif defined HAVE_X11_XPM_H
#include <X11/xpm.h>
#endif
#include "pixmap.h"
#include "globals.h"

#ifdef HAVE_IMLIB
static ImlibData *pID = NULL;
#endif

void InitPixmap()
{
#ifdef HAVE_IMLIB
  ImlibInitParams params;
  
  if (pID == NULL)
  {
    /* disable caching */
    params.flags = PARAMS_HIQUALITY | PARAMS_IMAGECACHESIZE | PARAMS_PIXMAPCACHESIZE | PARAMS_SHAREDPIXMAPS;
    params.hiquality = 1;
    params.sharedpixmaps = 1;
    params.imagecachesize = 0;
    params.pixmapcachesize = 0;
    
    pID = Imlib_init_with_params(pDisp,&params);
  }
#endif
}

void DestroyPixmap()
{

}

int LoadPixmap(char **pData,LPPIXMAP pMap)
{
#ifdef HAVE_IMLIB
  Imlib_data_to_pixmap(pID,pData,&pMap->icon,&pMap->mask);
#elif defined HAVE_XPMCREATEPIXMAPFROMDATA
  /* use the Xpm stuff */
  XpmAttributes attr;

  memset(&attr,0,sizeof(attr));

  XpmCreatePixmapFromData(pDisp,mainWindow,pData,&pMap->icon,&pMap->mask,&attr);
#else
  /* no image loader available */
  pMap->icon = None;
  pMap->mask = None;
#endif

  UpdatePixmapAttributes(pMap);
  return (pMap->icon ? 1: 0);
}

int LoadPixmapFromFile(const char *pFile,LPPIXMAP pMap)
{
#ifdef HAVE_IMLIB
  Imlib_load_file_to_pixmap(pID,(char*)pFile,&pMap->icon,&pMap->mask);
#elif defined HAVE_XPMREADFILETOPIXMAP
  /* use the Xpm stuff */
  XpmAttributes attr;

  memset(&attr,0,sizeof(attr));
  
  XpmReadFileToPixmap(pDisp,mainWindow,(char*)pFile,&pMap->icon,&pMap->mask,&attr);
#else
  /* no image loader available */
  pMap->icon = None;
  pMap->mask = None;
#endif
  /*printf("Icon/mask 0x%08X 0x%08X\n",pMap->icon,pMap->mask); */
  UpdatePixmapAttributes(pMap);

  return (pMap->icon ? 1 : 0);
}

void FreePixmap(LPPIXMAP pMap)
{
  if (pMap->icon != None)
  {
#ifdef HAVE_IMLIB
    Imlib_free_pixmap(pID,pMap->icon);
#else
    XFreePixmap(pDisp,pMap->icon);
#endif
    pMap->icon = None;
  }
  
  if (pMap->mask != None)
  {
#ifdef HAVE_IMLIB
    Imlib_free_pixmap(pID,pMap->mask);
#else
    XFreePixmap(pDisp,pMap->mask);
#endif
    pMap->mask = None;
  }
}

void UpdatePixmapAttributes(LPPIXMAP pMap)
{
  Window root;
  int x,y;
  unsigned int width,height,border,depth;
  
  if (pMap->icon != None)
  {
    XGetGeometry(pDisp,pMap->icon,&root,&x,&y,&width,&height,&border,&depth);
    pMap->width  = width;
    pMap->height = height;
  }
  else
  {
    pMap->width = 0;
    pMap->height = 0;
  }
}

void RenderPixmap(Display *pDisp,Drawable draw,GC gc,int x,int y,LPPIXMAP pMap)
{
  XSetClipMask(pDisp,gc,pMap->mask);
  XSetClipOrigin(pDisp,gc,x,y);
  XCopyArea(pDisp,pMap->icon,draw,gc,0,0,pMap->width,pMap->height,x,y);
  XSetClipMask(pDisp,gc,None);
}

/*int ScalePixmap(Display *display, Window window, LPPIXMAP src,LPPIXMAP dst)
{
  int x = 1;
  ImlibImage *pImage,*pScale;

  pImage = Imlib_create_image_from_drawable(pID,src->icon,src->mask,0,0,src->width,src->height);
  if (pImage)
  {
    pScale = Imlib_clone_scaled_image(pID,pImage,dst->width,dst->height);
    if (pScale)
    {
      x = 0;		       
      Imlib_render(pID,pScale,dst->width,dst->height);
      dst->icon = Imlib_move_image(pID,pScale);
      dst->mask = Imlib_move_mask(pID,pScale);
      
      Imlib_destroy_image(pID,pScale);
    }
    Imlib_destroy_image(pID,pImage);
  }
  return 0;
} */

int ScalePixmap(Display *display, Window window, LPPIXMAP src,LPPIXMAP dst)
{
  Pixmap 	temp, dest, tempMask, destMask;
  int 		j, end;
  float 	i;
  float 	xscale, yscale;
  XGCValues     gcv;
  GC            gccopy = gGc;
  GC            gcmask;
  
  xscale = (float)src->width / (float)dst->width;         /* Scaling factors */
  yscale = (float)src->height / (float)dst->height;
  
  dest = XCreatePixmap(display, window, dst->width, dst->height, 
		       DefaultDepth(display, XDefaultScreen(display)));
  temp = XCreatePixmap(display, window, dst->width, src->height, 
		       DefaultDepth(display, XDefaultScreen(display)));
  
  if (src->mask != None)
  {
    destMask = XCreatePixmap(display,window,dst->width,dst->height,1);
    tempMask = XCreatePixmap(display,window,dst->width,src->height,1);
    gcv.subwindow_mode = IncludeInferiors;
    gcv.graphics_exposures = False;
    gcmask = XCreateGC(display,destMask,GCGraphicsExposures | GCSubwindowMode,&gcv);
  }
  else
  {
    destMask = None;
    tempMask = None;
    gcmask = NULL;
  }
  
  j = 0;
  end = (int)(dst->width * xscale);
  
  /* Scale width of source into temp pixmap */
  for(i = 0; i < end; i += xscale, ++j)
  {
    XCopyArea(display, src->icon, temp, gccopy, 
	      (int) i, 0, 1, src->height, j, 0);

    if (tempMask != None)
    {
      XCopyArea(display, src->mask, tempMask, gcmask,
		(int)i, 0, 1, src->height, j, 0);
    }
  }
  
  j = 0;
  end = (int)(dst->height * yscale);
  
  /* Scale height of temp into dest pixmap */
  for(i = 0; i < end; i += yscale, ++j)
  {
    XCopyArea(display, temp, dest, gccopy, 
	      0, (int) i, dst->width, 1, 0, j);

    if (destMask != None)
    {
      XCopyArea(display, tempMask, destMask, gcmask,
		0, (int) i, dst->width, 1, 0, j);
    }
  }
  /* free up the pixmaps */
  XFreePixmap(display, temp);
  
  if (tempMask != None)
  {
    XFreePixmap(display,tempMask);
  }
  /* free up the extra GC */
  if (gcmask != NULL)
  {
    XFreeGC(display,gcmask); 
  }
  /* return results */
  dst->icon = dest;
  dst->mask = destMask;
  return 1;
}

