#include "config.h"
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <string.h>
#include "atoms.h"
#include "pixmap.h"
#include "globals.h"

void *GetPropertyData(Window win, Atom prop, Atom type, int *items)
{
  Atom type_ret;
  int format_ret;
  unsigned long items_ret;
  unsigned long after_ret;
  unsigned char *prop_data;
  
  prop_data = 0;
  
  XGetWindowProperty (pDisp, win, prop, 0, 0x7fffffff, False,
		      type, &type_ret, &format_ret, &items_ret,
		      &after_ret, &prop_data);
  if (items)
  {
    *items = items_ret;
  }
  
  return prop_data;
}

int GetInt(Window win, Atom at)
{
  int num = 0;
  unsigned long *data;
  
  data = (unsigned long*)GetPropertyData(win, at, XA_CARDINAL, 0);
  if (data)
  {
    num = *data;
    XFree (data);
  }
  return num;
}

void SetProperty(Window win,Atom at,Atom type,long val)
{
  XChangeProperty(pDisp,win,at,type,32,PropModeReplace,(unsigned char*)&val,1);
}

unsigned long GetColor(unsigned short red,unsigned short green,unsigned short blue)
{
  XColor xcl;
 
  xcl.red = red;
  xcl.green = green;
  xcl.blue = blue;

  XAllocColor(pDisp,DefaultColormap(pDisp,gScreen),&xcl);
  return xcl.pixel;
}

int GetDesktop(Window window)
{
  return GetInt(window,atom__NET_WM_DESKTOP);
}

int GetCurrentDesktop()
{
  return GetInt(root,atom__NET_CURRENT_DESKTOP);
}

int IsHidden(Window win)
{
  unsigned long *data;
  int ret = 0;
  int num;
  int i;

  data = (unsigned long*)GetPropertyData(win, atom__NET_WM_STATE, XA_ATOM, &num);
  if (data)
  {
    for (i=0;i<num;++i)
    {
      if (data[i] == atom__NET_WM_STATE_SKIP_TASKBAR)
      {
	ret = 1;
	break;
      }
    }
    XFree(data);
  }

  return ret;
}

int IsIconified(Window win)
{
  unsigned long *data;
  int ret = 0;
  
  data = (unsigned long*)GetPropertyData(win, atom__WM_STATE, atom__WM_STATE, 0);
  if (data)
  {
    if (*data == IconicState)
    {
      ret = 1;
    }
    XFree (data);
  }
  return ret;
}

void Maximize(Window wnd)
{
  XMapWindow(pDisp,wnd);
}

void Minimize(Window wnd)
{
  XIconifyWindow(pDisp,wnd,gScreen);
}

void SetFocus(Window wnd)
{
  XRaiseWindow(pDisp,wnd);
  XSetInputFocus(pDisp,wnd,RevertToPointerRoot,CurrentTime);
}

void ToggleShade(Window win)
{
  XClientMessageEvent xev;
  
  xev.type = ClientMessage;
  xev.window = win;
  xev.message_type = atom__NET_WM_STATE;
  xev.format = 32;
  xev.data.l[0] = 2;	/* toggle */
  xev.data.l[1] = atom__NET_WM_STATE_SHADED;
  xev.data.l[2] = 0;
  XSendEvent (pDisp, root, False, SubstructureNotifyMask, (XEvent *) &xev);
}

void PathRemove(char *buffer)
{
  char *found = strrchr(buffer,'/');
  if (found)
  {
    *found = 0;
  }
}

void PathAppend(char *buffer,const char *path)
{
  int len = strlen(buffer);
  if (buffer[len-1] != '/')
  {
    buffer[len] = '/';
    strcpy(buffer+len+1,path);
  }
  else
  {
    strcpy(buffer+len,path);
  }
}

char *PathGetExtension(char *buffer)
{
  char *found = strrchr(buffer,'.');
  return found;
}

int GetWindowPid(Window win)
{
  return GetInt(win,atom__NET_WM_PID);
}

void ClientToScreen(Window from,int x,int y,int *retX,int *retY)
{
  Window child;
  XTranslateCoordinates(pDisp,from,root,x,y,retX,retY,&child);
}

char *GetWindowName(Window win,int *length)
{
  XTextProperty prop;
  char **pList;
  int n;
  char *pRet = NULL;

  /* now fetch the name */
  if (XGetWMName(pDisp,win,&prop))
  {
    if (prop.value)
    {
      if (prop.nitems > 0)
      {
	if (prop.encoding == XA_STRING)
	{
	  pRet = strdup(prop.value);
	}
	else			/* not XA_STRING */
	{
	  if (XmbTextPropertyToTextList(pDisp,&prop,&pList,&n) >= Success &&
	      pList)
	  {
	    if ((n > 0) && *pList)
	    {
	      pRet = strdup(*pList);
	      prop.nitems = strlen(*pList);
	    }
	    XFreeStringList(pList);
	  }
	}
      }
      XFree(prop.value);
    }
  }
  /* update the length */
  *length = prop.nitems;
  /* return the correct value */
  return pRet;
}
